@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,28 +14,28 @@ public final class CallbackI extends _CallbackDisp
     public void
     initiateCallback(CallbackReceiverPrx proxy, Ice.Current current)
     {
-        System.out.println("initiating callback");
-	try
-	{
-	    proxy.callback(current.ctx);
-	}
-	catch(Ice.LocalException ex)
-	{
-	    ex.printStackTrace();
-	}
+        System.out.println("initiating callback to: " + current.adapter.getCommunicator().proxyToString(proxy));
+        try
+        {
+            proxy.callback(current.ctx);
+        }
+        catch(Ice.LocalException ex)
+        {
+            ex.printStackTrace();
+        }
     }
 
     public void
     shutdown(Ice.Current current)
     {
         System.out.println("Shutting down...");
-	try
-	{
-	    current.adapter.getCommunicator().shutdown();
-	}
-	catch(Ice.LocalException ex)
-	{
-	    ex.printStackTrace();
-	}
+        try
+        {
+            current.adapter.getCommunicator().shutdown();
+        }
+        catch(Ice.LocalException ex)
+        {
+            ex.printStackTrace();
+        }
     }
 }
