@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -58,28 +58,40 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isA(String __id)
     {
-        return ice_isA(__id, _reference.getContext());
+        return ice_isA(__id, null, false);
     }
 
     public final boolean
     ice_isA(String __id, java.util.Map __context)
     {
+        return ice_isA(__id, __context, true);
+    }
+
+    private boolean
+    ice_isA(String __id, java.util.Map __context, boolean __explicitCtx)
+    {
+        if(__explicitCtx && __context == null)
+        {
+            __context = _emptyContext;
+        }
+        
         int __cnt = 0;
         while(true)
         {
+            _ObjectDel __del = null;
             try
             {
-	        __checkTwowayOnly("ice_isA");
-                _ObjectDel __del = __getDelegate();
+                __checkTwowayOnly("ice_isA");
+                __del = __getDelegate();
                 return __del.ice_isA(__id, __context);
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__del, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__del, __ex, __cnt);
             }
         }
     }
@@ -87,87 +99,122 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final void
     ice_ping()
     {
-        ice_ping(_reference.getContext());
+        ice_ping(null, false);
     }
 
     public final void
     ice_ping(java.util.Map __context)
     {
+        ice_ping(__context, true);
+    }
+
+    private void
+    ice_ping(java.util.Map __context, boolean __explicitCtx)
+    {
+        if(__explicitCtx && __context == null)
+        {
+            __context = _emptyContext;
+        }
+
         int __cnt = 0;
         while(true)
         {
+            _ObjectDel __del = null;
             try
             {
-	        __checkTwowayOnly("ice_ping");
-                _ObjectDel __del = __getDelegate();
+                __del = __getDelegate();
                 __del.ice_ping(__context);
                 return;
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__del, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__del, __ex, __cnt);
             }
         }
     }
 
-    public String[]
+    public final String[]
     ice_ids()
     {
-        return ice_ids(_reference.getContext());
+        return ice_ids(null, false);
     }
 
-    public String[]
+    public final String[]
     ice_ids(java.util.Map __context)
     {
+        return ice_ids(__context, true);
+    }
+
+    private String[]
+    ice_ids(java.util.Map __context, boolean __explicitCtx)
+    {
+        if(__explicitCtx && __context == null)
+        {
+            __context = _emptyContext;
+        }
+
         int __cnt = 0;
         while(true)
         {
+            _ObjectDel __del = null;
             try
             {
-	        __checkTwowayOnly("ice_ids");
-                _ObjectDel __del = __getDelegate();
+                __checkTwowayOnly("ice_ids");
+                __del = __getDelegate();
                 return __del.ice_ids(__context);
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__del, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__del, __ex, __cnt);
             }
         }
     }
 
-    public String
+    public final String
     ice_id()
     {
-        return ice_id(_reference.getContext());
+        return ice_id(null, false);
     }
 
-    public String
+    public final String
     ice_id(java.util.Map __context)
     {
+        return ice_id(__context, true);
+    }
+
+    private String
+    ice_id(java.util.Map __context, boolean __explicitCtx)
+    {
+        if(__explicitCtx && __context == null)
+        {
+            __context = _emptyContext;
+        }
+
         int __cnt = 0;
         while(true)
         {
+            _ObjectDel __del = null;
             try
             {
-	        __checkTwowayOnly("ice_id");
-                _ObjectDel __del = __getDelegate();
+                __checkTwowayOnly("ice_id");
+                __del = __getDelegate();
                 return __del.ice_id(__context);
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
-                __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                __cnt = __handleExceptionWrapperRelaxed(__del, __ex, __cnt);
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__del, __ex, __cnt);
             }
         }
     }
@@ -175,35 +222,48 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_invoke(String operation, OperationMode mode, byte[] inParams, ByteSeqHolder outParams)
     {
-        return ice_invoke(operation, mode, inParams, outParams, _reference.getContext());
+        return ice_invoke(operation, mode, inParams, outParams, null, false);
     }
 
     public final boolean
     ice_invoke(String operation, OperationMode mode, byte[] inParams, ByteSeqHolder outParams,
                java.util.Map context)
     {
+        return ice_invoke(operation, mode, inParams, outParams, context, true);
+    }
+    
+    private boolean
+    ice_invoke(String operation, OperationMode mode, byte[] inParams, ByteSeqHolder outParams,
+               java.util.Map context, boolean explicitCtx)
+    {
+        if(explicitCtx && context == null)
+        {
+            context = _emptyContext;
+        }
+
         int __cnt = 0;
         while(true)
         {
+            _ObjectDel __del = null;
             try
             {
-                _ObjectDel __del = __getDelegate();
+                __del = __getDelegate();
                 return __del.ice_invoke(operation, mode, inParams, outParams, context);
             }
             catch(IceInternal.LocalExceptionWrapper __ex)
             {
                 if(mode == OperationMode.Nonmutating || mode == OperationMode.Idempotent)
                 {
-                    __cnt = __handleExceptionWrapperRelaxed(__ex, __cnt);
+                    __cnt = __handleExceptionWrapperRelaxed(__del, __ex, __cnt);
                 }
                 else
                 {
-                    __handleExceptionWrapper(__ex);
+                    __handleExceptionWrapper(__del, __ex);
                 }
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__del, __ex, __cnt);
             }
         }
     }
@@ -211,17 +271,22 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final void
     ice_invoke_async(AMI_Object_ice_invoke cb, String operation, OperationMode mode, byte[] inParams)
     {
-        ice_invoke_async(cb, operation, mode, inParams, _reference.getContext());
+        __checkTwowayOnly("ice_invoke_async");
+        cb.__invoke(this, operation, mode, inParams, null);
     }
 
     public final void
     ice_invoke_async(AMI_Object_ice_invoke cb, String operation, OperationMode mode, byte[] inParams,
-		     java.util.Map context)
+                     java.util.Map context)
     {
-	__checkTwowayOnly("ice_invoke_async");
-	cb.__invoke(this, operation, mode, inParams, context);
+        if(context == null)
+        {
+            context = _emptyContext;
+        }
+        __checkTwowayOnly("ice_invoke_async");
+        cb.__invoke(this, operation, mode, inParams, context);
     }
-
+    
     public final Identity
     ice_getIdentity()
     {
@@ -232,9 +297,9 @@ public class ObjectPrxHelperBase implements ObjectPrx
     ice_identity(Identity newIdentity)
     {
         if(newIdentity.name.equals(""))
-	{
-	    throw new IllegalIdentityException();
-	}
+        {
+            throw new IllegalIdentityException();
+        }
         if(newIdentity.equals(_reference.getIdentity()))
         {
             return this;
@@ -253,7 +318,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_newIdentity(Identity newIdentity)
     {
-	return ice_identity(newIdentity);
+        return ice_identity(newIdentity);
     }
 
     public final java.util.Map
@@ -265,9 +330,9 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_context(java.util.Map newContext)
     {
-	ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-	proxy.setup(_reference.changeContext(newContext));
-	return proxy;
+        ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+        proxy.setup(_reference.changeContext(newContext));
+        return proxy;
     }
 
     /**
@@ -276,15 +341,18 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_newContext(java.util.Map newContext)
     {
-	return ice_context(newContext);
+        return ice_context(newContext);
     }
 
+    /**
+     * @deprecated
+     **/
     public final ObjectPrx
     ice_defaultContext()
     {
-	ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-	proxy.setup(_reference.defaultContext());
-	return proxy;
+        ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+        proxy.setup(_reference.defaultContext());
+        return proxy;
     }
 
     public final String
@@ -319,7 +387,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_newFacet(String newFacet)
     {
-	return ice_facet(newFacet);
+        return ice_facet(newFacet);
     }
 
     public final String
@@ -331,6 +399,11 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_adapterId(String newAdapterId)
     {
+        if(newAdapterId == null)
+        {
+            newAdapterId = "";
+        }
+
         if(newAdapterId.equals(_reference.getAdapterId()))
         {
             return this;
@@ -349,7 +422,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_newAdapterId(String newAdapterId)
     {
-	return ice_adapterId(newAdapterId);
+        return ice_adapterId(newAdapterId);
     }
 
     public final Endpoint[]
@@ -367,8 +440,8 @@ public class ObjectPrxHelperBase implements ObjectPrx
         }
         else
         {
-	    IceInternal.EndpointI[] edpts = new IceInternal.EndpointI[newEndpoints.length];
-	    edpts = (IceInternal.EndpointI[])java.util.Arrays.asList(newEndpoints).toArray(edpts);
+            IceInternal.EndpointI[] edpts = new IceInternal.EndpointI[newEndpoints.length];
+            edpts = (IceInternal.EndpointI[])java.util.Arrays.asList(newEndpoints).toArray(edpts);
             ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
             proxy.setup(_reference.changeEndpoints(edpts));
             return proxy;
@@ -381,13 +454,13 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final ObjectPrx
     ice_newEndpoints(Endpoint[] newEndpoints)
     {
-	return ice_endpoints(newEndpoints);
+        return ice_endpoints(newEndpoints);
     }
 
     public final int
     ice_getLocatorCacheTimeout()
     {
-	return _reference.getLocatorCacheTimeout();
+        return _reference.getLocatorCacheTimeout();
     }
 
     public final ObjectPrx
@@ -408,7 +481,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isConnectionCached()
     {
-	return _reference.getCacheConnection();
+        return _reference.getCacheConnection();
     }
 
     public final ObjectPrx
@@ -429,7 +502,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final Ice.EndpointSelectionType
     ice_getEndpointSelection()
     {
-	return _reference.getEndpointSelection();
+        return _reference.getEndpointSelection();
     }
 
     public final ObjectPrx
@@ -450,7 +523,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isSecure()
     {
-	return _reference.getSecure();
+        return _reference.getSecure();
     }
 
     public final ObjectPrx
@@ -468,11 +541,32 @@ public class ObjectPrxHelperBase implements ObjectPrx
         }
     }
 
+    public final boolean
+    ice_isPreferSecure()
+    {
+        return _reference.getPreferSecure();
+    }
+
+    public final ObjectPrx
+    ice_preferSecure(boolean b)
+    {
+        if(b == _reference.getPreferSecure())
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(_reference.changePreferSecure(b));
+            return proxy;
+        }
+    }
+
     public final Ice.RouterPrx
     ice_getRouter()
     {
-	IceInternal.RouterInfo ri = _reference.getRouterInfo();
-	return ri != null ? ri.getRouter() : null;
+        IceInternal.RouterInfo ri = _reference.getRouterInfo();
+        return ri != null ? ri.getRouter() : null;
     }
 
     public final ObjectPrx
@@ -494,8 +588,8 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final Ice.LocatorPrx
     ice_getLocator()
     {
-	IceInternal.LocatorInfo ri = _reference.getLocatorInfo();
-	return ri != null ? ri.getLocator() : null;
+        IceInternal.LocatorInfo ri = _reference.getLocatorInfo();
+        return ri != null ? ri.getLocator() : null;
     }
 
     public final ObjectPrx
@@ -517,7 +611,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isCollocationOptimized()
     {
-	return _reference.getCollocationOptimization();
+        return _reference.getCollocationOptimization();
     }
 
     /**
@@ -562,7 +656,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isTwoway()
     {
-	return _reference.getMode() == IceInternal.Reference.ModeTwoway;
+        return _reference.getMode() == IceInternal.Reference.ModeTwoway;
     }
 
     public final ObjectPrx
@@ -583,7 +677,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isOneway()
     {
-	return _reference.getMode() == IceInternal.Reference.ModeOneway;
+        return _reference.getMode() == IceInternal.Reference.ModeOneway;
     }
 
     public final ObjectPrx
@@ -604,7 +698,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isBatchOneway()
     {
-	return _reference.getMode() == IceInternal.Reference.ModeBatchOneway;
+        return _reference.getMode() == IceInternal.Reference.ModeBatchOneway;
     }
 
     public final ObjectPrx
@@ -625,7 +719,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isDatagram()
     {
-	return _reference.getMode() == IceInternal.Reference.ModeDatagram;
+        return _reference.getMode() == IceInternal.Reference.ModeDatagram;
     }
 
     public final ObjectPrx
@@ -646,7 +740,7 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final boolean
     ice_isBatchDatagram()
     {
-	return _reference.getMode() == IceInternal.Reference.ModeBatchDatagram;
+        return _reference.getMode() == IceInternal.Reference.ModeBatchDatagram;
     }
 
     public final ObjectPrx
@@ -697,6 +791,28 @@ public class ObjectPrxHelperBase implements ObjectPrx
         }
     }
 
+    public boolean
+    ice_isThreadPerConnection()
+    {
+        return _reference.getThreadPerConnection();
+    }
+
+    public ObjectPrx
+    ice_threadPerConnection(boolean tpc)
+    {
+        IceInternal.Reference ref = _reference.changeThreadPerConnection(tpc);
+        if(ref.equals(_reference))
+        {
+            return this;
+        }
+        else
+        {
+            ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+            proxy.setup(ref);
+            return proxy;
+        }
+    }
+
     /**
      * @deprecated This method has been replaced with ice_collocationOptimized.
      **/
@@ -712,32 +828,55 @@ public class ObjectPrxHelperBase implements ObjectPrx
         int __cnt = 0;
         while(true)
         {
+            _ObjectDel __del = null;
             try
             {
-                _ObjectDel __del = __getDelegate();
+                __del = __getDelegate();
                 return __del.__getConnection(new BooleanHolder());
             }
             catch(LocalException __ex)
             {
-                __cnt = __handleException(__ex, __cnt);
+                __cnt = __handleException(__del, __ex, __cnt);
             }
         }
     }
 
+    public final Connection
+    ice_getCachedConnection()
+    {
+        _ObjectDel __del = null;
+        synchronized(this)
+        {
+            __del = _delegate;
+        }
+        
+        if(__del != null)
+        {
+            try
+            {
+                return __del.__getConnection(new BooleanHolder());
+            }
+            catch(CollocationOptimizationException ex)
+            {
+            }
+        }
+        return null;
+    }
+
     public final boolean
     equals(java.lang.Object r)
     {
-	if(this == r)
-	{
-	    return true;
-	}
+        if(this == r)
+        {
+            return true;
+        }
 
-	if(r instanceof ObjectPrxHelperBase)
-	{
-	    return _reference.equals(((ObjectPrxHelperBase)r)._reference);
-	}
+        if(r instanceof ObjectPrxHelperBase)
+        {
+            return _reference.equals(((ObjectPrxHelperBase)r)._reference);
+        }
 
-	return false;
+        return false;
     }
 
     public final IceInternal.Reference
@@ -778,66 +917,74 @@ public class ObjectPrxHelperBase implements ObjectPrx
         // called upon initialization.
         //
 
-	assert(_reference == null);
-	assert(_delegate == null);
+        assert(_reference == null);
+        assert(_delegate == null);
 
         _reference = ref;
 
-	if(_reference.getCacheConnection())
-	{
-	    //
-	    // The _delegate attribute is only used if "cache connection"
-	    // is enabled. If it's not enabled, we don't keep track of the
-	    // delegate -- a new delegate is created for each invocations.
-	    //	
-	    
-	    if(delegateD != null)
-	    {
-		_ObjectDelD delegate = __createDelegateD();
-		delegate.__copyFrom(delegateD);
-		_delegate = delegate;
-	    }
-	    else if(delegateM != null)
-	    {
-		_ObjectDelM delegate = __createDelegateM();
-		delegate.__copyFrom(delegateM);
-		_delegate = delegate;
-	    }
-	}
+        if(_reference.getCacheConnection())
+        {
+            //
+            // The _delegate attribute is only used if "cache connection"
+            // is enabled. If it's not enabled, we don't keep track of the
+            // delegate -- a new delegate is created for each invocations.
+            //  
+            
+            if(delegateD != null)
+            {
+                _ObjectDelD delegate = __createDelegateD();
+                delegate.__copyFrom(delegateD);
+                _delegate = delegate;
+            }
+            else if(delegateM != null)
+            {
+                _ObjectDelM delegate = __createDelegateM();
+                delegate.__copyFrom(delegateM);
+                _delegate = delegate;
+            }
+        }
     }
 
     public final int
-    __handleException(LocalException ex, int cnt)
+    __handleException(_ObjectDel delegate, LocalException ex, int cnt)
     {
-	//
-	// Only _delegate needs to be mutex protected here.
-	//
-	synchronized(this)
-	{
-	    _delegate = null;
-	}
+        //
+        // Only _delegate needs to be mutex protected here.
+        //
+        synchronized(this)
+        {
+            if(delegate == _delegate)
+            {
+                _delegate = null;
+            }
+        }
 
-	IceInternal.ProxyFactory proxyFactory = _reference.getInstance().proxyFactory();
-	if(proxyFactory != null)
-	{
-	    return proxyFactory.checkRetryAfterException(ex, _reference, cnt);
-	}
-	else
-	{
-	    //
+        IceInternal.ProxyFactory proxyFactory;
+        try
+        {
+            proxyFactory = _reference.getInstance().proxyFactory();
+        }
+        catch(CommunicatorDestroyedException e)
+        {
+            //
             // The communicator is already destroyed, so we cannot
             // retry.
-	    //
-	    throw ex;
-	}
+            //
+            throw ex;
+        }
+
+        return proxyFactory.checkRetryAfterException(ex, _reference, cnt);
     }
 
     public final void
-    __handleExceptionWrapper(IceInternal.LocalExceptionWrapper ex)
+    __handleExceptionWrapper(_ObjectDel delegate, IceInternal.LocalExceptionWrapper ex)
     {
         synchronized(this)
         {
-            _delegate = null;
+            if(delegate == _delegate)
+            {
+                _delegate = null;
+            }
         }
 
         if(!ex.retry())
@@ -847,17 +994,20 @@ public class ObjectPrxHelperBase implements ObjectPrx
     }
 
     public final int
-    __handleExceptionWrapperRelaxed(IceInternal.LocalExceptionWrapper ex, int cnt)
+    __handleExceptionWrapperRelaxed(_ObjectDel delegate, IceInternal.LocalExceptionWrapper ex, int cnt)
     {
         if(!ex.retry())
         {
-            return __handleException(ex.get(), cnt);
+            return __handleException(delegate, ex.get(), cnt);
         }
         else
         {
             synchronized(this)
             {
-                _delegate = null;
+                if(delegate == _delegate)
+                {
+                    _delegate = null;
+                }
             }
             return cnt;
         }
@@ -866,17 +1016,17 @@ public class ObjectPrxHelperBase implements ObjectPrx
     public final void
     __checkTwowayOnly(String name)
     {
-	//
-	// No mutex lock necessary, there is nothing mutable in this
-	// operation.
-	//
+        //
+        // No mutex lock necessary, there is nothing mutable in this
+        // operation.
+        //
 
         if(!ice_isTwoway())
-	{
-	    TwowayOnlyException ex = new TwowayOnlyException();
-	    ex.operation = name;
-	    throw ex;
-	}
+        {
+            TwowayOnlyException ex = new TwowayOnlyException();
+            ex.operation = name;
+            throw ex;
+        }
     }
 
     public final synchronized _ObjectDel
@@ -884,50 +1034,50 @@ public class ObjectPrxHelperBase implements ObjectPrx
     {
         if(_delegate != null)
         {
-	    return _delegate;
-	}
+            return _delegate;
+        }
 
-	_ObjectDel delegate = null;
-	if(_reference.getCollocationOptimization())
-	{
-	    ObjectAdapter adapter = _reference.getInstance().objectAdapterFactory().findObjectAdapter(this);
-	    if(adapter != null)
-	    {
-		_ObjectDelD d = __createDelegateD();
-		d.setup(_reference, adapter);
-		delegate = d;
-	    }
-	}
+        _ObjectDel delegate = null;
+        if(_reference.getCollocationOptimization())
+        {
+            ObjectAdapter adapter = _reference.getInstance().objectAdapterFactory().findObjectAdapter(this);
+            if(adapter != null)
+            {
+                _ObjectDelD d = __createDelegateD();
+                d.setup(_reference, adapter);
+                delegate = d;
+            }
+        }
 
-	if(delegate == null)
-	{
-	    _ObjectDelM d = __createDelegateM();
-	    d.setup(_reference);
-	    delegate = d;
+        if(delegate == null)
+        {
+            _ObjectDelM d = __createDelegateM();
+            d.setup(_reference);
+            delegate = d;
 
-	    //
-	    // If this proxy is for a non-local object, and we are
-	    // using a router, then add this proxy to the router info
-	    // object.
-	    //
-	    IceInternal.RouterInfo ri = _reference.getRouterInfo();
-	    if(ri != null)
-	    {
-		ri.addProxy(this);
-	    }
-	}
+            //
+            // If this proxy is for a non-local object, and we are
+            // using a router, then add this proxy to the router info
+            // object.
+            //
+            IceInternal.RouterInfo ri = _reference.getRouterInfo();
+            if(ri != null)
+            {
+                ri.addProxy(this);
+            }
+        }
 
-	if(_reference.getCacheConnection())
-	{
-	    //
-	    // The _delegate attribute is only used if "cache connection"
-	    // is enabled. If it's not enabled, we don't keep track of the
-	    // delegate -- a new delegate is created for each invocations.
-	    //
-	    _delegate = delegate;
-	}
+        if(_reference.getCacheConnection())
+        {
+            //
+            // The _delegate attribute is only used if "cache connection"
+            // is enabled. If it's not enabled, we don't keep track of the
+            // delegate -- a new delegate is created for each invocations.
+            //
+            _delegate = delegate;
+        }
 
-	return delegate;
+        return delegate;
     }
 
     protected _ObjectDelM
@@ -942,12 +1092,6 @@ public class ObjectPrxHelperBase implements ObjectPrx
         return new _ObjectDelD();
     }
 
-    protected java.util.Map
-    __defaultContext()
-    {
-        return _reference.getContext();
-    }
-
     //
     // Only for use by IceInternal.ProxyFactory
     //
@@ -959,12 +1103,14 @@ public class ObjectPrxHelperBase implements ObjectPrx
         // upon initial initialization.
         //
 
-	assert(_reference == null);
-	assert(_delegate == null);
+        assert(_reference == null);
+        assert(_delegate == null);
 
         _reference = ref;
     }
 
+    protected static final java.util.Map _emptyContext = new java.util.HashMap();
+
     private IceInternal.Reference _reference;
     private _ObjectDel _delegate;
 }
