@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,9 +17,13 @@ public final class ReferenceFactory
            String facet,
            int mode,
            boolean secure,
+           boolean preferSecure,
            EndpointI[] endpoints,
            RouterInfo routerInfo,
-	   boolean collocationOptimization)
+           boolean collocationOptimization,
+           boolean cacheConnection,
+           Ice.EndpointSelectionType endpointSelection,
+           boolean threadPerConnection)
     {
         if(_instance == null)
         {
@@ -35,8 +39,9 @@ public final class ReferenceFactory
         // Create new reference
         //
         DirectReference ref = new DirectReference(_instance, _communicator, ident, context, facet, mode, secure,
-						  endpoints, routerInfo, collocationOptimization);
-	return updateCache(ref);
+                                                  preferSecure, endpoints, routerInfo, collocationOptimization,
+                                                  cacheConnection, endpointSelection, threadPerConnection);
+        return updateCache(ref);
     }
 
     public synchronized Reference
@@ -45,11 +50,15 @@ public final class ReferenceFactory
            String facet,
            int mode,
            boolean secure,
+           boolean preferSecure,
            String adapterId,
            RouterInfo routerInfo,
-	   LocatorInfo locatorInfo,
-	   boolean collocationOptimization,
-	   int locatorCacheTimeout)
+           LocatorInfo locatorInfo,
+           boolean collocationOptimization,
+           boolean cacheConnection,
+           Ice.EndpointSelectionType endpointSelection,
+           boolean threadPerConnection,
+           int locatorCacheTimeout)
     {
         if(_instance == null)
         {
@@ -65,9 +74,10 @@ public final class ReferenceFactory
         // Create new reference
         //
         IndirectReference ref = new IndirectReference(_instance, _communicator, ident, context, facet, mode, secure,
-						      adapterId, routerInfo, locatorInfo, collocationOptimization,
-						      locatorCacheTimeout);
-	return updateCache(ref);
+                                                      preferSecure, adapterId, routerInfo, locatorInfo,
+                                                      collocationOptimization, cacheConnection, endpointSelection,
+                                                      threadPerConnection, locatorCacheTimeout);
+        return updateCache(ref);
     }
 
     public synchronized Reference
@@ -75,7 +85,7 @@ public final class ReferenceFactory
            java.util.Map context,
            String facet,
            int mode,
-	   Ice.ConnectionI[] fixedConnections)
+           Ice.ConnectionI[] fixedConnections)
     {
         if(_instance == null)
         {
@@ -91,24 +101,24 @@ public final class ReferenceFactory
         // Create new reference
         //
         FixedReference ref = new FixedReference(_instance, _communicator, ident, context, facet, mode,
-						fixedConnections);
-	return updateCache(ref);
+                                                fixedConnections);
+        return updateCache(ref);
     }
 
     public synchronized Reference
     copy(Reference r)
     {
         if(_instance == null)
-	{
-	    throw new Ice.CommunicatorDestroyedException();
-	}
+        {
+            throw new Ice.CommunicatorDestroyedException();
+        }
 
-	Ice.Identity ident = r.getIdentity();
-	if(ident.name.length() == 0 && ident.category.length() == 0)
-	{
-	    return null;
-	}
-	return (Reference)r.clone();
+        Ice.Identity ident = r.getIdentity();
+        if(ident.name.length() == 0 && ident.category.length() == 0)
+        {
+            return null;
+        }
+        return (Reference)r.clone();
     }
 
     public Reference
@@ -128,8 +138,8 @@ public final class ReferenceFactory
         if(beg == -1)
         {
             Ice.ProxyParseException e = new Ice.ProxyParseException();
-	    e.str = s;
-	    throw e;
+            e.str = s;
+            throw e;
         }
 
         //
@@ -141,8 +151,8 @@ public final class ReferenceFactory
         if(end == -1)
         {
             Ice.ProxyParseException e = new Ice.ProxyParseException();
-	    e.str = s;
-	    throw e;
+            e.str = s;
+            throw e;
         }
         else if(end == 0)
         {
@@ -163,8 +173,8 @@ public final class ReferenceFactory
         if(beg == end)
         {
             Ice.ProxyParseException e = new Ice.ProxyParseException();
-	    e.str = s;
-	    throw e;
+            e.str = s;
+            throw e;
         }
 
         //
@@ -205,7 +215,7 @@ public final class ReferenceFactory
         String facet = "";
         int mode = Reference.ModeTwoway;
         boolean secure = false;
-	String adapter = "";
+        String adapter = "";
 
         while(true)
         {
@@ -235,8 +245,8 @@ public final class ReferenceFactory
             if(option.length() != 2 || option.charAt(0) != '-')
             {
                 Ice.ProxyParseException e = new Ice.ProxyParseException();
-		e.str = s;
-		throw e;
+                e.str = s;
+                throw e;
             }
 
             //
@@ -256,8 +266,8 @@ public final class ReferenceFactory
                     if(end == -1)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     else if(end == 0)
                     {
@@ -288,16 +298,16 @@ public final class ReferenceFactory
                     if(argument == null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
 
                     Ice.StringHolder facetH = new Ice.StringHolder();
                     if(!IceUtil.StringUtil.unescapeString(argument, 0, argument.length(), facetH))
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
 
                     facet = facetH.value;
@@ -309,8 +319,8 @@ public final class ReferenceFactory
                     if(argument != null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     mode = Reference.ModeTwoway;
                     break;
@@ -321,8 +331,8 @@ public final class ReferenceFactory
                     if(argument != null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     mode = Reference.ModeOneway;
                     break;
@@ -333,8 +343,8 @@ public final class ReferenceFactory
                     if(argument != null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     mode = Reference.ModeBatchOneway;
                     break;
@@ -345,8 +355,8 @@ public final class ReferenceFactory
                     if(argument != null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     mode = Reference.ModeDatagram;
                     break;
@@ -357,8 +367,8 @@ public final class ReferenceFactory
                     if(argument != null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     mode = Reference.ModeBatchDatagram;
                     break;
@@ -369,8 +379,8 @@ public final class ReferenceFactory
                     if(argument != null)
                     {
                         Ice.ProxyParseException e = new Ice.ProxyParseException();
-			e.str = s;
-			throw e;
+                        e.str = s;
+                        throw e;
                     }
                     secure = true;
                     break;
@@ -379,120 +389,243 @@ public final class ReferenceFactory
                 default:
                 {
                     Ice.ProxyParseException e = new Ice.ProxyParseException();
-		    e.str = s;
-		    throw e;
+                    e.str = s;
+                    throw e;
                 }
             }
         }
 
-	RouterInfo routerInfo = _instance.routerManager().get(getDefaultRouter());
-	LocatorInfo locatorInfo = _instance.locatorManager().get(getDefaultLocator());
+        RouterInfo routerInfo = _instance.routerManager().get(getDefaultRouter());
+        LocatorInfo locatorInfo = _instance.locatorManager().get(getDefaultLocator());
 
-	if(beg == -1)
-	{
-	    return create(ident, _instance.getDefaultContext(), facet, mode, secure, "", routerInfo,
-	    		  locatorInfo, _instance.defaultsAndOverrides().defaultCollocationOptimization,
-			  _instance.defaultsAndOverrides().defaultLocatorCacheTimeout);
-	}
+        if(beg == -1)
+        {
+            return create(ident, _instance.getDefaultContext(), facet, mode, secure,
+                          _instance.defaultsAndOverrides().defaultPreferSecure, "", routerInfo,
+                          locatorInfo, _instance.defaultsAndOverrides().defaultCollocationOptimization, true,
+                          _instance.defaultsAndOverrides().defaultEndpointSelection, _instance.threadPerConnection(),
+                          _instance.defaultsAndOverrides().defaultLocatorCacheTimeout);
+        }
 
         java.util.ArrayList endpoints = new java.util.ArrayList();
 
-	if(s.charAt(beg) == ':')
-	{
-	    java.util.ArrayList unknownEndpoints = new java.util.ArrayList();
-	    end = beg;
+        if(s.charAt(beg) == ':')
+        {
+            java.util.ArrayList unknownEndpoints = new java.util.ArrayList();
+            end = beg;
 
-	    while(end < s.length() && s.charAt(end) == ':')
-	    {
-		beg = end + 1;
-		
-		end = s.indexOf(':', beg);
-		if(end == -1)
-		{
-		    end = s.length();
-		}
-		
-		String es = s.substring(beg, end);
-		EndpointI endp = _instance.endpointFactoryManager().create(es);
-		if(endp != null)
-		{
-		    java.util.ArrayList endps = endp.expand(false);
-		    endpoints.addAll(endps);
-		}
-		else
-		{
-		    unknownEndpoints.add(es);
-		}
-	    }
-	    if(endpoints.size() == 0)
-	    {
-	        Ice.EndpointParseException e = new Ice.EndpointParseException();
-		e.str = (String)unknownEndpoints.get(0);
-		throw e;
-	    }
-	    else if(unknownEndpoints.size() != 0 &&
-	           _instance.initializationData().properties.getPropertyAsIntWithDefault("Ice.Warn.Endpoints", 1) > 0)
-	    {
-	        String msg = "Proxy contains unknown endpoints:";
-		java.util.Iterator iter = unknownEndpoints.iterator();
-		while(iter.hasNext())
-		{
-		    msg += " `" + (String)iter.next() + "'";
-		}
-		_instance.initializationData().logger.warning(msg);
-	    }
+            while(end < s.length() && s.charAt(end) == ':')
+            {
+                beg = end + 1;
+                
+                end = s.indexOf(':', beg);
+                if(end == -1)
+                {
+                    end = s.length();
+                }
+                
+                String es = s.substring(beg, end);
+                EndpointI endp = _instance.endpointFactoryManager().create(es);
+                if(endp != null)
+                {
+                    java.util.ArrayList endps = endp.expand(false);
+                    endpoints.addAll(endps);
+                }
+                else
+                {
+                    unknownEndpoints.add(es);
+                }
+            }
+            if(endpoints.size() == 0)
+            {
+                Ice.EndpointParseException e = new Ice.EndpointParseException();
+                e.str = (String)unknownEndpoints.get(0);
+                throw e;
+            }
+            else if(unknownEndpoints.size() != 0 &&
+                   _instance.initializationData().properties.getPropertyAsIntWithDefault("Ice.Warn.Endpoints", 1) > 0)
+            {
+                String msg = "Proxy contains unknown endpoints:";
+                java.util.Iterator iter = unknownEndpoints.iterator();
+                while(iter.hasNext())
+                {
+                    msg += " `" + (String)iter.next() + "'";
+                }
+                _instance.initializationData().logger.warning(msg);
+            }
 
-	    EndpointI[] endp = new EndpointI[endpoints.size()];
-	    endpoints.toArray(endp);
-	    return create(ident, _instance.getDefaultContext(), facet, mode, secure, endp, routerInfo,
-			  _instance.defaultsAndOverrides().defaultCollocationOptimization);
-	}
-	else if(s.charAt(beg) == '@')
-	{
-	    beg = IceUtil.StringUtil.findFirstNotOf(s, delim, beg + 1);
-	    if(beg == -1)
-	    {
-		Ice.ProxyParseException e = new Ice.ProxyParseException();
-		e.str = s;
-		throw e;
-	    }
+            EndpointI[] endp = new EndpointI[endpoints.size()];
+            endpoints.toArray(endp);
+            return create(ident, _instance.getDefaultContext(), facet, mode, secure, 
+                          _instance.defaultsAndOverrides().defaultPreferSecure, endp, routerInfo,
+                          _instance.defaultsAndOverrides().defaultCollocationOptimization, true,
+                          _instance.defaultsAndOverrides().defaultEndpointSelection, _instance.threadPerConnection());
+        }
+        else if(s.charAt(beg) == '@')
+        {
+            beg = IceUtil.StringUtil.findFirstNotOf(s, delim, beg + 1);
+            if(beg == -1)
+            {
+                Ice.ProxyParseException e = new Ice.ProxyParseException();
+                e.str = s;
+                throw e;
+            }
 
-	    end = IceUtil.StringUtil.checkQuote(s, beg);
-	    if(end == -1)
-	    {
-		Ice.ProxyParseException e = new Ice.ProxyParseException();
-		e.str = s;
-		throw e;
-	    }
-	    else if(end == 0)
-	    {
-		end = IceUtil.StringUtil.findFirstOf(s, delim, beg);
-		if(end == -1)
-		{
-		    end = s.length();
-		}
-	    }
-	    else
-	    {
-		beg++; // Skip leading quote
-	    }
+            String adapterstr = null;
+            end = IceUtil.StringUtil.checkQuote(s, beg);
+            if(end == -1)
+            {
+                Ice.ProxyParseException e = new Ice.ProxyParseException();
+                e.str = s;
+                throw e;
+            }
+            else if(end == 0)
+            {
+                end = IceUtil.StringUtil.findFirstOf(s, delim, beg);
+                if(end == -1)
+                {
+                    end = s.length();
+                }
+                adapterstr = s.substring(beg, end);
+            }
+            else
+            {
+                beg++; // Skip leading quote
+                adapterstr = s.substring(beg, end);
+                end++; // Skip trailing quote
+            }
 
-	    Ice.StringHolder token = new Ice.StringHolder();
-	    if(!IceUtil.StringUtil.unescapeString(s, beg, end, token) || token.value.length() == 0)
-	    {
-		Ice.ProxyParseException e = new Ice.ProxyParseException();
-		e.str = s;
-		throw e;
-	    }
-	    adapter = token.value;
-	    return create(ident, _instance.getDefaultContext(), facet, mode, secure, adapter,
-	    		  routerInfo, locatorInfo, _instance.defaultsAndOverrides().defaultCollocationOptimization,
-			  _instance.defaultsAndOverrides().defaultLocatorCacheTimeout);
-	}
+            if(end != s.length() && IceUtil.StringUtil.findFirstNotOf(s, delim, end) != -1)
+            {
+                Ice.ProxyParseException e = new Ice.ProxyParseException();
+                e.str = s;
+                throw e;
+            }
+            
+            Ice.StringHolder token = new Ice.StringHolder();
+            if(!IceUtil.StringUtil.unescapeString(adapterstr, 0, adapterstr.length(), token) ||
+               token.value.length() == 0)
+            {
+                Ice.ProxyParseException e = new Ice.ProxyParseException();
+                e.str = s;
+                throw e;
+            }
+            adapter = token.value;
+            return create(ident, _instance.getDefaultContext(), facet, mode, secure,
+                          _instance.defaultsAndOverrides().defaultPreferSecure, adapter,
+                          routerInfo, locatorInfo, _instance.defaultsAndOverrides().defaultCollocationOptimization,
+                          true, _instance.defaultsAndOverrides().defaultEndpointSelection,
+                          _instance.threadPerConnection(), _instance.defaultsAndOverrides().defaultLocatorCacheTimeout);
+        }
 
-	Ice.ProxyParseException ex = new Ice.ProxyParseException();
-	ex.str = s;
-	throw ex;
+        Ice.ProxyParseException ex = new Ice.ProxyParseException();
+        ex.str = s;
+        throw ex;
+    }
+
+    public Reference
+    createFromProperties(String propertyPrefix)
+    {
+        Ice.Properties properties = _instance.initializationData().properties;
+
+        Reference ref = create(properties.getProperty(propertyPrefix));
+        if(ref == null)
+        {
+            return null;
+        }
+
+        //
+        // Warn about unknown properties.
+        //
+        if(properties.getPropertyAsIntWithDefault("Ice.Warn.UnknownProperties", 1) > 0)
+        {
+            checkForUnknownProperties(propertyPrefix);
+        }
+
+        String property = propertyPrefix + ".Locator";
+        if(properties.getProperty(property).length() != 0)
+        {
+            ref = ref.changeLocator(Ice.LocatorPrxHelper.uncheckedCast(_communicator.propertyToProxy(property)));
+            if(ref instanceof DirectReference)
+            {
+                String s = "`" + property + "=" + properties.getProperty(property) +
+                           "': cannot set a locator on a direct reference; setting ignored";
+                _instance.initializationData().logger.warning(s);
+            }
+        }
+
+        property = propertyPrefix + ".LocatorCacheTimeout";
+        if(properties.getProperty(property).length() != 0)
+        {
+            ref = ref.changeLocatorCacheTimeout(properties.getPropertyAsInt(property));
+            if(ref instanceof DirectReference)
+            {
+                String s = "`" + property + "=" + properties.getProperty(property) +
+                           "': cannot set a locator cache timeout on a direct reference; setting ignored";
+                _instance.initializationData().logger.warning(s);
+            }
+        }
+
+        property = propertyPrefix + ".Router";
+        if(properties.getProperty(property).length() != 0)
+        {
+            if(propertyPrefix.endsWith(".Router"))
+            {
+                String s = "`" + property + "=" + properties.getProperty(property) +
+                           "': cannot set a router on a router; setting ignored";
+                _instance.initializationData().logger.warning(s);
+            }
+            else
+            {
+                ref = ref.changeRouter(Ice.RouterPrxHelper.uncheckedCast(_communicator.propertyToProxy(property)));
+            }
+        }
+
+        property = propertyPrefix + ".PreferSecure";
+        if(properties.getProperty(property).length() != 0)
+        {
+            ref = ref.changePreferSecure(properties.getPropertyAsInt(property) > 0);
+        }
+
+        property = propertyPrefix + ".ConnectionCached";
+        if(properties.getProperty(property).length() != 0)
+        {
+            ref = ref.changeCacheConnection(properties.getPropertyAsInt(property) > 0);
+        }
+
+        property = propertyPrefix + ".EndpointSelection";
+        if(properties.getProperty(property).length() != 0)
+        {
+            String type = properties.getProperty(property);
+            if(type.equals("Random"))
+            {
+                ref = ref.changeEndpointSelection(Ice.EndpointSelectionType.Random);
+            }
+            else if(type.equals("Ordered"))
+            {
+                ref = ref.changeEndpointSelection(Ice.EndpointSelectionType.Ordered);
+            }
+            else
+            {
+                Ice.EndpointSelectionTypeParseException ex = new Ice.EndpointSelectionTypeParseException();
+                ex.str = type;
+                throw ex;
+            }
+        }
+
+        property = propertyPrefix + ".CollocationOptimization";
+        if(properties.getProperty(property).length() != 0)
+        {
+            ref = ref.changeCollocationOptimization(properties.getPropertyAsInt(property) > 0);
+        }
+
+        property = propertyPrefix + ".ThreadPerConnection";
+        if(properties.getProperty(property).length() != 0)
+        {
+            ref = ref.changeThreadPerConnection(properties.getPropertyAsInt(property) > 0);
+        }
+
+        return ref;
     }
 
     public Reference
@@ -515,16 +648,16 @@ public final class ReferenceFactory
         String facet;
         if(facetPath.length > 0)
         {
-	    if(facetPath.length > 1)
-	    {
-	        throw new Ice.ProxyUnmarshalException();
-	    }
+            if(facetPath.length > 1)
+            {
+                throw new Ice.ProxyUnmarshalException();
+            }
             facet = facetPath[0];
         }
-	else
+        else
         {
-	    facet = "";
-	}
+            facet = "";
+        }
 
         int mode = (int)s.readByte();
         if(mode < 0 || mode > Reference.ModeLast)
@@ -535,31 +668,34 @@ public final class ReferenceFactory
         boolean secure = s.readBool();
 
         EndpointI[] endpoints;
-	String adapterId = "";
+        String adapterId = "";
 
         RouterInfo routerInfo = _instance.routerManager().get(getDefaultRouter());
         LocatorInfo locatorInfo = _instance.locatorManager().get(getDefaultLocator());
 
         int sz = s.readSize();
-	if(sz > 0)
-	{
-	    endpoints = new EndpointI[sz];
-	    for(int i = 0; i < sz; i++)
-	    {
-		endpoints[i] = _instance.endpointFactoryManager().read(s);
-	    }
-	    return create(ident, _instance.getDefaultContext(), facet, mode, secure, endpoints,
-	    		  routerInfo, _instance.defaultsAndOverrides().defaultCollocationOptimization);
-	}
-	else
-	{
-	    endpoints = new EndpointI[0];
-	    adapterId = s.readString();
-	    return create(ident, _instance.getDefaultContext(), facet, mode, secure,
-	                  adapterId, routerInfo, locatorInfo,
-			  _instance.defaultsAndOverrides().defaultCollocationOptimization,
-			  _instance.defaultsAndOverrides().defaultLocatorCacheTimeout);
-	}
+        if(sz > 0)
+        {
+            endpoints = new EndpointI[sz];
+            for(int i = 0; i < sz; i++)
+            {
+                endpoints[i] = _instance.endpointFactoryManager().read(s);
+            }
+            return create(ident, _instance.getDefaultContext(), facet, mode, secure, 
+                          _instance.defaultsAndOverrides().defaultPreferSecure, endpoints,
+                          routerInfo, _instance.defaultsAndOverrides().defaultCollocationOptimization, true,
+                          _instance.defaultsAndOverrides().defaultEndpointSelection, _instance.threadPerConnection());
+        }
+        else
+        {
+            endpoints = new EndpointI[0];
+            adapterId = s.readString();
+            return create(ident, _instance.getDefaultContext(), facet, mode, secure,
+                          _instance.defaultsAndOverrides().defaultPreferSecure, adapterId, routerInfo, locatorInfo,
+                          _instance.defaultsAndOverrides().defaultCollocationOptimization, true,
+                          _instance.defaultsAndOverrides().defaultEndpointSelection, _instance.threadPerConnection(),
+                          _instance.defaultsAndOverrides().defaultLocatorCacheTimeout);
+        }
     }
 
     public synchronized void
@@ -645,6 +781,68 @@ public final class ReferenceFactory
         return ref;
     }
 
+    static private String[] _suffixes =
+    {
+        "EndpointSelection",
+        "ConnectionCached",
+        "PreferSecure",
+        "LocatorCacheTimeout",
+        "Locator",
+        "Router",
+        "CollocationOptimization",
+        "ThreadPerConnection"
+    };
+
+    private void
+    checkForUnknownProperties(String prefix)
+    {
+        //
+        // Do not warn about unknown properties if Ice prefix, ie Ice, Glacier2, etc
+        //      
+        for(int i = 0; IceInternal.PropertyNames.clPropNames[i] != null; ++i)
+        {
+            if(prefix.startsWith(IceInternal.PropertyNames.clPropNames[i] + "."))
+            {
+                return;
+            }
+        }
+
+        java.util.ArrayList unknownProps = new java.util.ArrayList();
+        java.util.Map props = _instance.initializationData().properties.getPropertiesForPrefix(prefix + ".");
+        java.util.Iterator p = props.entrySet().iterator();
+        while(p.hasNext())
+        {
+            java.util.Map.Entry entry = (java.util.Map.Entry)p.next();
+            String prop = (String)entry.getKey();
+
+            boolean valid = false;
+            for(int i = 0; i < _suffixes.length; ++i)
+            {
+                if(prop.equals(prefix + "." + _suffixes[i]))
+                {
+                    valid = true;
+                    break;
+                }
+            }
+
+            if(!valid)
+            {
+                unknownProps.add(prop);
+            }
+        }
+
+        if(unknownProps.size() != 0)
+        {
+            String message = "found unknown properties for proxy '" + prefix + "':";
+            p = unknownProps.iterator();
+            while(p.hasNext())
+            {
+                message += "\n    " + (String)p.next();
+            }
+            _instance.initializationData().logger.warning(message);
+        }
+    }
+
     private Instance _instance;
     private Ice.Communicator _communicator;
     private Ice.RouterPrx _defaultRouter;
