@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,14 +19,14 @@ final class UdpEndpointI extends EndpointI
         _instance = instance;
         _host = ho;
         _port = po;
-	_protocolMajor = Protocol.protocolMajor;
-	_protocolMinor = Protocol.protocolMinor;
-	_encodingMajor = Protocol.encodingMajor;
-	_encodingMinor = Protocol.encodingMinor;
+        _protocolMajor = Protocol.protocolMajor;
+        _protocolMinor = Protocol.protocolMinor;
+        _encodingMajor = Protocol.encodingMajor;
+        _encodingMinor = Protocol.encodingMinor;
         _connect = conn;
-	_connectionId = conId;
-	_compress = co;
-	_publish = pub;
+        _connectionId = conId;
+        _compress = co;
+        _publish = pub;
         calcHashValue();
     }
 
@@ -36,13 +36,13 @@ final class UdpEndpointI extends EndpointI
         _instance = instance;
         _host = null;
         _port = 0;
-	_protocolMajor = Protocol.protocolMajor;
-	_protocolMinor = Protocol.protocolMinor;
-	_encodingMajor = Protocol.encodingMajor;
-	_encodingMinor = Protocol.encodingMinor;
+        _protocolMajor = Protocol.protocolMajor;
+        _protocolMinor = Protocol.protocolMinor;
+        _encodingMajor = Protocol.encodingMajor;
+        _encodingMinor = Protocol.encodingMinor;
         _connect = false;
-	_compress = false;
-	_publish = true;
+        _compress = false;
+        _publish = true;
 
         String[] arr = str.split("[ \t\n\r]+");
 
@@ -58,7 +58,7 @@ final class UdpEndpointI extends EndpointI
             String option = arr[i++];
             if(option.length() != 2 || option.charAt(0) != '-')
             {
-		throw new Ice.EndpointParseException("udp " + str);
+                throw new Ice.EndpointParseException("udp " + str);
             }
 
             String argument = null;
@@ -73,46 +73,46 @@ final class UdpEndpointI extends EndpointI
                 {
                     if(argument == null)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
-		    int pos = argument.indexOf('.');
+                    int pos = argument.indexOf('.');
                     if(pos == -1)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
-		    String majStr = argument.substring(0, pos);
-		    String minStr = argument.substring(pos + 1, argument.length());
-		    int majVersion;
-		    int minVersion;
-		    try
-		    {
-		        majVersion = Integer.parseInt(majStr);
-			minVersion = Integer.parseInt(minStr);
-		    }
+                    String majStr = argument.substring(0, pos);
+                    String minStr = argument.substring(pos + 1, argument.length());
+                    int majVersion;
+                    int minVersion;
+                    try
+                    {
+                        majVersion = Integer.parseInt(majStr);
+                        minVersion = Integer.parseInt(minStr);
+                    }
                     catch(NumberFormatException ex)
-		    {
-			throw new Ice.EndpointParseException("udp " + str);
-		    }
+                    {
+                        throw new Ice.EndpointParseException("udp " + str);
+                    }
 
-		    if(majVersion < 1 || majVersion > 255 || minVersion < 0 || minVersion > 255)
-		    {
-			throw new Ice.EndpointParseException("udp " + str);
-		    }
+                    if(majVersion < 1 || majVersion > 255 || minVersion < 0 || minVersion > 255)
+                    {
+                        throw new Ice.EndpointParseException("udp " + str);
+                    }
 
-		    if(majVersion != Protocol.protocolMajor)
-		    {
-			Ice.UnsupportedProtocolException e = new Ice.UnsupportedProtocolException();
-			e.badMajor = majVersion < 0 ? majVersion + 255 : majVersion;
-			e.badMinor = minVersion < 0 ? minVersion + 255 : minVersion;
-			e.major = Protocol.protocolMajor;
-			e.minor = Protocol.protocolMinor;
-			throw e;
-		    }
+                    if(majVersion != Protocol.protocolMajor)
+                    {
+                        Ice.UnsupportedProtocolException e = new Ice.UnsupportedProtocolException();
+                        e.badMajor = majVersion < 0 ? majVersion + 255 : majVersion;
+                        e.badMinor = minVersion < 0 ? minVersion + 255 : minVersion;
+                        e.major = Protocol.protocolMajor;
+                        e.minor = Protocol.protocolMinor;
+                        throw e;
+                    }
 
-		    _protocolMajor = (byte)majVersion;
-		    _protocolMinor = (byte)minVersion;
+                    _protocolMajor = (byte)majVersion;
+                    _protocolMinor = (byte)minVersion;
 
                     break;
                 }
@@ -121,46 +121,46 @@ final class UdpEndpointI extends EndpointI
                 {
                     if(argument == null)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
-		    int pos = argument.indexOf('.');
+                    int pos = argument.indexOf('.');
                     if(pos == -1)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
-		    String majStr = argument.substring(0, pos);
-		    String minStr = argument.substring(pos + 1, argument.length());
-		    int majVersion;
-		    int minVersion;
-		    try
-		    {
-		        majVersion = Integer.parseInt(majStr);
-			minVersion = Integer.parseInt(minStr);
-		    }
+                    String majStr = argument.substring(0, pos);
+                    String minStr = argument.substring(pos + 1, argument.length());
+                    int majVersion;
+                    int minVersion;
+                    try
+                    {
+                        majVersion = Integer.parseInt(majStr);
+                        minVersion = Integer.parseInt(minStr);
+                    }
                     catch(NumberFormatException ex)
-		    {
-			throw new Ice.EndpointParseException("udp " + str);
-		    }
+                    {
+                        throw new Ice.EndpointParseException("udp " + str);
+                    }
 
-		    if(majVersion < 1 || majVersion > 255 || minVersion < 0 || minVersion > 255)
-		    {
-			throw new Ice.EndpointParseException("udp " + str);
-		    }
+                    if(majVersion < 1 || majVersion > 255 || minVersion < 0 || minVersion > 255)
+                    {
+                        throw new Ice.EndpointParseException("udp " + str);
+                    }
 
-		    if(majVersion != Protocol.encodingMajor)
-		    {
-			Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
-			e.badMajor = majVersion < 0 ? majVersion + 255 : majVersion;
-			e.badMinor = minVersion < 0 ? minVersion + 255 : minVersion;
-			e.major = Protocol.encodingMajor;
-			e.minor = Protocol.encodingMinor;
-			throw e;
-		    }
+                    if(majVersion != Protocol.encodingMajor)
+                    {
+                        Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
+                        e.badMajor = majVersion < 0 ? majVersion + 255 : majVersion;
+                        e.badMinor = minVersion < 0 ? minVersion + 255 : minVersion;
+                        e.major = Protocol.encodingMajor;
+                        e.minor = Protocol.encodingMinor;
+                        throw e;
+                    }
 
-		    _encodingMajor = (byte)majVersion;
-		    _encodingMinor = (byte)minVersion;
+                    _encodingMajor = (byte)majVersion;
+                    _encodingMinor = (byte)minVersion;
 
                     break;
                 }
@@ -169,7 +169,7 @@ final class UdpEndpointI extends EndpointI
                 {
                     if(argument == null)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
                     _host = argument;
@@ -180,7 +180,7 @@ final class UdpEndpointI extends EndpointI
                 {
                     if(argument == null)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
                     try
@@ -189,13 +189,13 @@ final class UdpEndpointI extends EndpointI
                     }
                     catch(NumberFormatException ex)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
-		    if(_port < 0 || _port > 65535)
-		    {
-			throw new Ice.EndpointParseException("udp " + str);
-		    }
+                    if(_port < 0 || _port > 65535)
+                    {
+                        throw new Ice.EndpointParseException("udp " + str);
+                    }
 
                     break;
                 }
@@ -204,7 +204,7 @@ final class UdpEndpointI extends EndpointI
                 {
                     if(argument != null)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
                     _connect = true;
@@ -215,7 +215,7 @@ final class UdpEndpointI extends EndpointI
                 {
                     if(argument != null)
                     {
-			throw new Ice.EndpointParseException("udp " + str);
+                        throw new Ice.EndpointParseException("udp " + str);
                     }
 
                     _compress = true;
@@ -224,25 +224,10 @@ final class UdpEndpointI extends EndpointI
 
                 default:
                 {
-		    throw new Ice.EndpointParseException("udp " + str);
+                    throw new Ice.EndpointParseException("udp " + str);
                 }
             }
         }
-
-        if(_host == null)
-        {
-            _host = instance.defaultsAndOverrides().defaultHost;
-            if(_host == null)
-            {
-		_host = "0.0.0.0";
-            }
-        }
-	else if(_host.equals("*"))
-	{
-	    _host = "0.0.0.0";
-	}
-
-        calcHashValue();
     }
 
     public
@@ -252,34 +237,34 @@ final class UdpEndpointI extends EndpointI
         s.startReadEncaps();
         _host = s.readString();
         _port = s.readInt();
-	_protocolMajor = s.readByte();
-	_protocolMinor = s.readByte();
-	_encodingMajor = s.readByte();
-	_encodingMinor = s.readByte();
-	if(_protocolMajor != Protocol.protocolMajor)
-	{
-	    Ice.UnsupportedProtocolException e = new Ice.UnsupportedProtocolException();
-	    e.badMajor = _protocolMajor < 0 ? _protocolMajor + 255 : _protocolMajor;
-	    e.badMinor = _protocolMinor < 0 ? _protocolMinor + 255 : _protocolMinor;
-	    e.major = Protocol.protocolMajor;
-	    e.minor = Protocol.protocolMinor;
-	    throw e;
-	}
-	if(_encodingMajor != Protocol.encodingMajor)
-	{
-	    Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
-	    e.badMajor = _encodingMajor < 0 ? _encodingMajor + 255 : _encodingMajor;
-	    e.badMinor = _encodingMinor < 0 ? _encodingMinor + 255 : _encodingMinor;
-	    e.major = Protocol.encodingMajor;
-	    e.minor = Protocol.encodingMinor;
-	    throw e;
-	}
+        _protocolMajor = s.readByte();
+        _protocolMinor = s.readByte();
+        _encodingMajor = s.readByte();
+        _encodingMinor = s.readByte();
+        if(_protocolMajor != Protocol.protocolMajor)
+        {
+            Ice.UnsupportedProtocolException e = new Ice.UnsupportedProtocolException();
+            e.badMajor = _protocolMajor < 0 ? _protocolMajor + 255 : _protocolMajor;
+            e.badMinor = _protocolMinor < 0 ? _protocolMinor + 255 : _protocolMinor;
+            e.major = Protocol.protocolMajor;
+            e.minor = Protocol.protocolMinor;
+            throw e;
+        }
+        if(_encodingMajor != Protocol.encodingMajor)
+        {
+            Ice.UnsupportedEncodingException e = new Ice.UnsupportedEncodingException();
+            e.badMajor = _encodingMajor < 0 ? _encodingMajor + 255 : _encodingMajor;
+            e.badMinor = _encodingMinor < 0 ? _encodingMinor + 255 : _encodingMinor;
+            e.major = Protocol.encodingMajor;
+            e.minor = Protocol.encodingMinor;
+            throw e;
+        }
         // Not transmitted.
         //_connect = s.readBool();
         _connect = false;
-	_compress = s.readBool();
+        _compress = s.readBool();
         s.endReadEncaps();
-	_publish = true;
+        _publish = true;
         calcHashValue();
     }
 
@@ -293,13 +278,13 @@ final class UdpEndpointI extends EndpointI
         s.startWriteEncaps();
         s.writeString(_host);
         s.writeInt(_port);
-	s.writeByte(_protocolMajor);
-	s.writeByte(_protocolMinor);
-	s.writeByte(_encodingMajor);
-	s.writeByte(_encodingMinor);
+        s.writeByte(_protocolMajor);
+        s.writeByte(_protocolMinor);
+        s.writeByte(_encodingMajor);
+        s.writeByte(_encodingMinor);
         // Not transmitted.
         //s.writeBool(_connect);
-	s.writeBool(_compress);
+        s.writeBool(_compress);
         s.endWriteEncaps();
     }
 
@@ -309,38 +294,38 @@ final class UdpEndpointI extends EndpointI
     public String
     _toString()
     {
-	//
-	// WARNING: Certain features, such as proxy validation in Glacier2,
-	// depend on the format of proxy strings. Changes to toString() and
-	// methods called to generate parts of the reference string could break
-	// these features. Please review for all features that depend on the
-	// format of proxyToString() before changing this and related code.
-	//
+        //
+        // WARNING: Certain features, such as proxy validation in Glacier2,
+        // depend on the format of proxy strings. Changes to toString() and
+        // methods called to generate parts of the reference string could break
+        // these features. Please review for all features that depend on the
+        // format of proxyToString() before changing this and related code.
+        //
         String s = "udp";
 
-	if((int)_protocolMajor != 1 || (int)_protocolMinor != 0)
-	{
-	    s += " -v " + (_protocolMajor < 0 ? (int)_protocolMajor + 255 : _protocolMajor)
-		+ "." + (_protocolMinor < 0 ? (int)_protocolMinor + 255 : _protocolMinor);
-	}
+        if((int)_protocolMajor != 1 || (int)_protocolMinor != 0)
+        {
+            s += " -v " + (_protocolMajor < 0 ? (int)_protocolMajor + 255 : _protocolMajor)
+                + "." + (_protocolMinor < 0 ? (int)_protocolMinor + 255 : _protocolMinor);
+        }
 
-	if((int)_encodingMajor != 1 || (int)_encodingMinor != 0)
-	{
-	    s += " -e " + (_encodingMajor < 0 ? (int)_encodingMajor + 255 : _encodingMajor)
-		+ "." + (_encodingMinor < 0 ? (int)_encodingMinor + 255 : _encodingMinor);
-	}
+        if((int)_encodingMajor != 1 || (int)_encodingMinor != 0)
+        {
+            s += " -e " + (_encodingMajor < 0 ? (int)_encodingMajor + 255 : _encodingMajor)
+                + "." + (_encodingMinor < 0 ? (int)_encodingMinor + 255 : _encodingMinor);
+        }
 
-	s += " -h " + _host + " -p " + _port;
+        s += " -h " + _host + " -p " + _port;
 
         if(_connect)
         {
             s += " -c";
         }
 
-	if(_compress)
-	{
-	    s += " -z";
-	}
+        if(_compress)
+        {
+            s += " -z";
+        }
 
         return s;
     }
@@ -398,7 +383,7 @@ final class UdpEndpointI extends EndpointI
     public EndpointI
     connectionId(String connectionId)
     {
-        if(connectionId == _connectionId)
+        if(connectionId.equals(_connectionId))
         {
             return this;
         }
@@ -468,7 +453,7 @@ final class UdpEndpointI extends EndpointI
     {
         UdpTransceiver p = new UdpTransceiver(_instance, _host, _port, _connect);
         endpoint.value = new UdpEndpointI(_instance, _host, p.effectivePort(), _connect, _connectionId, _compress,
-					  _publish);
+                                          _publish);
         return p;
     }
 
@@ -502,8 +487,28 @@ final class UdpEndpointI extends EndpointI
     // only applies for ObjectAdapter endpoints.
     //
     public java.util.ArrayList
-    expand(boolean includeLoopback)
+    expand(boolean server)
     {
+        if(_host == null)
+        {
+            _host = _instance.defaultsAndOverrides().defaultHost;
+            if(_host == null)
+            {
+                if(server)
+                {
+                    _host = "0.0.0.0";
+                }
+                else
+                {
+                    _host = "127.0.0.1";
+                }
+            }
+        }
+        else if(_host.equals("*"))
+        {
+            _host = "0.0.0.0";
+        }
+
         java.util.ArrayList endps = new java.util.ArrayList();
         if(_host.equals("0.0.0.0"))
         {
@@ -512,15 +517,13 @@ final class UdpEndpointI extends EndpointI
             while(iter.hasNext())
             {
                 String host = (String)iter.next();
-		if(includeLoopback || hosts.size() == 1 || !host.equals("127.0.0.1"))
-		{
-                    endps.add(new UdpEndpointI(_instance, host, _port, _connect, _connectionId, _compress,
-					       hosts.size() == 1 || !host.equals("127.0.0.1")));
-		}
+                endps.add(new UdpEndpointI(_instance, host, _port, _connect, _connectionId, _compress,
+                                           hosts.size() == 1 || !host.equals("127.0.0.1")));
             }
         }
         else
         {
+            calcHashValue();
             endps.add(this);
         }
         return endps;
@@ -613,10 +616,10 @@ final class UdpEndpointI extends EndpointI
             return 1;
         }
 
-    	if(!_connectionId.equals(p._connectionId))
-	{
-	    return _connectionId.compareTo(p._connectionId);
-	}
+        if(!_connectionId.equals(p._connectionId))
+        {
+            return _connectionId.compareTo(p._connectionId);
+        }
 
         if(!_compress && p._compress)
         {
@@ -627,41 +630,41 @@ final class UdpEndpointI extends EndpointI
             return 1;
         }
 
-	if(_protocolMajor < p._protocolMajor)
-	{
-	    return -1;
-	}
-	else if(p._protocolMajor < _protocolMajor)
-	{
-	    return 1;
-	}
+        if(_protocolMajor < p._protocolMajor)
+        {
+            return -1;
+        }
+        else if(p._protocolMajor < _protocolMajor)
+        {
+            return 1;
+        }
 
-	if(_protocolMinor < p._protocolMinor)
-	{
-	    return -1;
-	}
-	else if(p._protocolMinor < _protocolMinor)
-	{
-	    return 1;
-	}
+        if(_protocolMinor < p._protocolMinor)
+        {
+            return -1;
+        }
+        else if(p._protocolMinor < _protocolMinor)
+        {
+            return 1;
+        }
 
-	if(_encodingMajor < p._encodingMajor)
-	{
-	    return -1;
-	}
-	else if(p._encodingMajor < _encodingMajor)
-	{
-	    return 1;
-	}
+        if(_encodingMajor < p._encodingMajor)
+        {
+            return -1;
+        }
+        else if(p._encodingMajor < _encodingMajor)
+        {
+            return 1;
+        }
 
-	if(_encodingMinor < p._encodingMinor)
-	{
-	    return -1;
-	}
-	else if(p._encodingMinor < _encodingMinor)
-	{
-	    return 1;
-	}
+        if(_encodingMinor < p._encodingMinor)
+        {
+            return -1;
+        }
+        else if(p._encodingMinor < _encodingMinor)
+        {
+            return 1;
+        }
 
         if(!_host.equals(p._host))
         {
@@ -669,62 +672,68 @@ final class UdpEndpointI extends EndpointI
             // We do the most time-consuming part of the comparison last.
             //
             java.net.InetSocketAddress laddr = null;
-	    try
-	    {
-		laddr = Network.getAddress(_host, _port);
-	    }
-	    catch(Ice.DNSException ex)
-	    {
-	    }
+            try
+            {
+                laddr = Network.getAddress(_host, _port);
+            }
+            catch(Ice.DNSException ex)
+            {
+            }
 
             java.net.InetSocketAddress raddr = null;
-	    try
-	    {
-		raddr = Network.getAddress(p._host, p._port);
-	    }
-	    catch(Ice.DNSException ex)
-	    {
-	    }
+            try
+            {
+                raddr = Network.getAddress(p._host, p._port);
+            }
+            catch(Ice.DNSException ex)
+            {
+            }
 
-	    if(laddr == null && raddr != null)
-	    {
-		return -1;
-	    }
-	    else if(raddr == null && laddr != null)
-	    {
-		return 1;
-	    }
-	    else if(laddr != null && raddr != null)
-	    {
-		byte[] larr = laddr.getAddress().getAddress();
-		byte[] rarr = raddr.getAddress().getAddress();
-		assert(larr.length == rarr.length);
-		for(int i = 0; i < larr.length; i++)
-		{
-		    if(larr[i] < rarr[i])
-		    {
-			return -1;
-		    }
-		    else if(rarr[i] < larr[i])
-		    {
-			return 1;
-		    }
-		}
-	    }
+            if(laddr == null && raddr != null)
+            {
+                return -1;
+            }
+            else if(raddr == null && laddr != null)
+            {
+                return 1;
+            }
+            else if(laddr != null && raddr != null)
+            {
+                byte[] larr = laddr.getAddress().getAddress();
+                byte[] rarr = raddr.getAddress().getAddress();
+                assert(larr.length == rarr.length);
+                for(int i = 0; i < larr.length; i++)
+                {
+                    if(larr[i] < rarr[i])
+                    {
+                        return -1;
+                    }
+                    else if(rarr[i] < larr[i])
+                    {
+                        return 1;
+                    }
+                }
+            }
         }
 
         return 0;
     }
 
+    public boolean
+    requiresThreadPerConnection()
+    {
+        return false;
+    }
+
     private void
     calcHashValue()
     {
         try
         {
-            java.net.InetAddress addr = java.net.InetAddress.getByName(_host);
-            _hashCode = addr.getHostAddress().hashCode();
+            java.net.InetSocketAddress addr = Network.getAddress(_host, _port);
+            _hashCode = addr.getAddress().getHostAddress().hashCode();
         }
-        catch(java.net.UnknownHostException ex)
+        catch(Ice.DNSException ex)
         {
             _hashCode = _host.hashCode();
         }
