@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,8 +19,7 @@ public final class TestI extends _TestIntfDisp
         Ice.ObjectAdapter adapter =
             communicator.createObjectAdapterWithEndpoints("TransientTestAdapter", "default -p 9999");
         adapter.activate();
-        adapter.deactivate();
-        adapter.waitForDeactivate();
+        adapter.destroy();
     }
 
     public void
