@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -32,7 +32,8 @@ os.environ["CLASSPATH"] = os.path.join(testdir, "classes") + TestUtil.sep + os.g
 #
 # Test client/server without on demand activation.
 #
-IceGridAdmin.iceGridClientServerTest(name, "", "--TestAdapter.Endpoints=default --TestAdapter.AdapterId=TestAdapter")
+IceGridAdmin.iceGridClientServerTest(name, "", "--TestAdapter.Endpoints=default" + \
+                                               " --TestAdapter.AdapterId=TestAdapter")
 
 #
 # Test client/server with on demand activation.
