@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -25,7 +25,7 @@ testdir = os.path.join(toplevel, "test", name)
 os.environ["CLASSPATH"] = os.path.join(testdir, "classes") + TestUtil.sep + os.getenv("CLASSPATH", "")
 
 print "starting client...",
-clientPipe = os.popen("java -ea Client 2>&1")
+clientPipe = os.popen(TestUtil.javaCmd + " -ea Client 2>&1")
 print "ok"
 
 TestUtil.printOutputFromPipe(clientPipe)
